==> 0.9.5.tar.gz: sinsy/src/lib/lyric/LyricTimelineESPEAK.cpp <==
#include "LyricTimelineESPEAK.h"
#include "util_log.h"
#include <boost/algorithm/string/classification.hpp>
#include <boost/algorithm/string/split.hpp>
#include <boost/lexical_cast.hpp>

// https://projectsonata.bandcamp.com
// https://golubovsky.bandcamp.com/track/clouds-of-anything

std::string pho(int i)
{
    char ret[32];
    sprintf(ret,"PHO[%i]",i);
    return ret;
}

// stretch vowel only to fill note, enough room for others
// enough room, no stretch needed anywhere, insert REST
// squish vowel, keep C and O same, insert REST

namespace sinsy {

ILyricTimeline* newTimelineESPEAK() { return new LyricTimelineESPEAK(); }

LyricTimelineESPEAK::LyricTimelineESPEAK()
{
    //TODO
}

static float oto_preutter(lyricPoint* p)
{
    if(p->paramsf.count("PreUtterance"))
        return p->paramsf["PreUtterance"];
    return 0;
}
static float oto_overlap(lyricPoint* p)
{
    if(p->paramsf.count("VoiceOverlap"))
        return p->paramsf["VoiceOverlap"];
    return 0;
}

void LyricTimelineESPEAK::fix()
{

}

static void doDefault(lyricPoint* p)
{
  float length_vowel = 0;
  float length_coda = 0;
  float length_onset = 0;
  int vowel_count = 0;
  
  if(p->pho_times.size()!=0) return;
  
  for(int i=0;i<256;i++)
  {
       std::string key = pho(i);
       std::string value = p->params[key];
       if(value=="") break;
       
       std::vector<std::string> spl;
       boost::split(spl, value, boost::is_any_of("\t "),
                   boost::token_compress_on);
       
       
       std::string code = spl[2];
       std::string type = spl[3];
       
       if(type=="PAUSE" && length_vowel>0) break;
       //LOG_MSG("oto_param: "<<value);
       
       float a = boost::lexical_cast<float>(spl[0]);
       p->pho_times.push_back(a);
       float b = boost::lexical_cast<float>(spl[1]);
       float length = (b-a);
       
       p->pho_lengths.push_back(length);
       p->pho_codes.push_back(code);
       p->pho_types.push_back(type);
       
       if(type=="VOWEL")
       {
            length_vowel+=length;
            vowel_count++;
       }
       else
       {
         if(length_vowel)
         {
             length_coda += length; 
         }
         else
         {
             length_onset += length;
         }
       }
       
       
    //LOG_MSG("len: "<<len);
       
  }
  //LOG_MSG("len o="<<length_onset<<" v="<<length_vowel<<" c="<<length_coda);
  p->paramsf["PreUtterance"] = length_onset;
  p->paramsf["DefVowel"] = length_vowel;
  p->paramsf["DefCoda"] = length_coda;
  p->paramsf["VowelCount"] = (float)vowel_count;
  
  uint sz = p->pho_times.size();
  p->pho_times.push_back(p->pho_times[sz-1] + p->pho_lengths[sz-1]);
  
}

void LyricTimelineESPEAK::emit(lyricPoint* cur,lyricPoint* next,IUnisyn* synth)
{
  // vowel, coda, next onset default durations (from phonemes)
  doDefault(cur);
  if(next) doDefault(next);
  //LOG_MSG("emit");
  
  float n_start = cur->times[0];
  float n_end =  cur->times.back();

  if (next) {
     n_end -= oto_preutter(next);
     n_end += oto_overlap(next);
  }
  
  n_start -= oto_preutter(cur);
  float n_len = n_end - n_start;
  
  //first caluclate rho
  
  //doTarget()
  
  //LOG_MSG("note: "<<n_start << " " <<n_end);
  float def_length = cur->paramsf["PreUtterance"]+cur->paramsf["DefVowel"]+cur->paramsf["DefCoda"];
  float vowel_count = cur->paramsf["VowelCount"];
  if(n_len > def_length)
  {
      float extend = n_len - def_length;
      LOG_MSG("extend "<<extend);
      
      float accu = n_start;
      cur->pho_output.push_back(accu);
      for(int i=0;i<cur->pho_lengths.size();i++)
      {
          float length2 = cur->pho_lengths[i];
          if(cur->pho_types[i]=="VOWEL")
          {
              length2 += extend/vowel_count;
          }
          accu += length2;
          cur->pho_output.push_back(accu);
      }
  }
  else
  {
      float compress = def_length/n_len;
      LOG_MSG("compress "<<compress);
      float accu = n_start;
      cur->pho_output.push_back(accu);
      int size=cur->pho_lengths.size();
      for(int i=0;i<size;i++)
      {
          float length2 = cur->pho_lengths[i];
          length2 -= compress/size;
          LOG_MSG("compress2 "<<length2);
          accu += length2;
          cur->pho_output.push_back(accu);
      }
  }
  
  int count1 = cur->pho_times.size();
  int count2 = cur->pho_output.size();
  
  
  
  auto valid = cur->params.count("FileName");
  
  if (valid && n_len > 0 && count1==count2) {
      LOG_MSG("emit: "<<count2);
      synth->addUnit(cur->params["FileName"], count2, cur->pho_output.data(), cur->pho_times.data());
  
  }
  //else LOG_ERR("LyricTimelineESPEAK::emit:: invalid note object");

}



bool LyricTimelineESPEAK::isValid(){
    return true;
}

void LyricTimelineESPEAK::outputPho(IUnisyn* synth)
{ 
    //first query output
    for(int i=0;i<points.size();i++)
    {
         auto p = points[i];
         auto fileName = p->params["FileName"];
         float length = synth->getLengthForUnit(fileName);
         
         for(int j=0;j<256;j++)
         {
             std::string value = synth->getPhoLine(fileName,j);
             if(value.length()==0) break;
             std::string key = pho(j);
             p->params[key]=value;
             
             
         }
         
         
         if(length>0) p->paramsf["Length"]=length;
         
    }
    
    LyricTimeline::outputPho(synth);
    
}


}

==> 0.9.5.3.tar.gz: sinsy/src/lib/lyric/LyricTimelineESPEAK.cpp <==
#include "LyricTimelineESPEAK.h"
#include "util_log.h"
#include <boost/algorithm/string/classification.hpp>
#include <boost/algorithm/string/split.hpp>
#include <boost/lexical_cast.hpp>

// https://projectsonata.bandcamp.com
// https://golubovsky.bandcamp.com/track/clouds-of-anything

std::string pho(int i)
{
    char ret[32];
    sprintf(ret,"PHO[%i]",i);
    return ret;
}

// stretch vowel only to fill note, enough room for others
// enough room, no stretch needed anywhere, insert REST
// squish vowel, keep C and O same, insert REST

namespace sinsy {

ILyricTimeline* newTimelineESPEAK() { return new LyricTimelineESPEAK(); }

LyricTimelineESPEAK::LyricTimelineESPEAK()
{
    //TODO
}

//unify preutter and overlap ??
static float oto_preutter(lyricPoint* p)
{
    if(p->paramsf.count("PreUtterance"))
        return p->paramsf["PreUtterance"];
    return 0;
}
static float oto_overlap(lyricPoint* p)
{
    if(p->paramsf.count("VoiceOverlap"))
        return p->paramsf["VoiceOverlap"];
    return 0;
}

void LyricTimelineESPEAK::fix()
{

}

static void doDefault(lyricPoint* p)
{
  float length_vowel = 0;
  float length_coda = 0;
  float length_onset = 0;
  int vowel_count = 0;
  
  if(p->pho_times.size()!=0) return;
  
  for(int i=0;i<256;i++)
  {
       std::string key = pho(i);
       std::string value = p->params[key];
       if(value=="") break;
       
       std::vector<std::string> spl;
       boost::split(spl, value, boost::is_any_of("\t "),
                   boost::token_compress_on);
       
       
       std::string code = spl[2];
       std::string type = spl[3];
       
       if(type=="PAUSE" && length_vowel>0) break;
       //LOG_MSG("oto_param: "<<value);
       
       float a = boost::lexical_cast<float>(spl[0]);
       p->pho_times.push_back(a);
       float b = boost::lexical_cast<float>(spl[1]);
       float length = (b-a);
       
       p->pho_lengths.push_back(length);
       p->pho_codes.push_back(code);
       p->pho_types.push_back(type);
       
       if(type=="VOWEL")
       {
            length_vowel+=length;
            vowel_count++;
       }
       else
       {
         if(length_vowel)
         {
             length_coda += length; 
         }
         else
         {
             length_onset += length;
         }
       }
       
       
    //LOG_MSG("len: "<<len);
       
  }
  //LOG_MSG("len o="<<length_onset<<" v="<<length_vowel<<" c="<<length_coda);
  p->paramsf["PreUtterance"] = length_onset;
  p->paramsf["DefVowel"] = length_vowel;
  p->paramsf["DefCoda"] = length_coda;
  p->paramsf["VowelCount"] = (float)vowel_count;
  
  uint sz = p->pho_times.size();
  assert(sz>=1);
  p->pho_times.push_back(p->pho_times[sz-1] + p->pho_lengths[sz-1]);
  
}

void LyricTimelineESPEAK::emit(lyricPoint* cur,lyricPoint* next,IUnisyn* synth)
{
  // calculate default length vowel, coda, next onset default durations (from phonemes)
  doDefault(cur);
  if(next) doDefault(next);
  
  float n_start = cur->times[0];
  float n_end =  cur->times.back();

  if (next) {
     n_end -= oto_preutter(next);
     n_end += oto_overlap(next);
  }
  
  n_start -= oto_preutter(cur);
  float n_len = n_end - n_start;
  
  //TODO: handle diphone voices (festival/mbrola)
  
  // change length of phonemes to fit not duration
  
  float def_length = cur->paramsf["PreUtterance"]+cur->paramsf["DefVowel"]+cur->paramsf["DefCoda"];
  float vowel_count = cur->paramsf["VowelCount"];
  if(n_len > def_length)
  {
      float extend = n_len - def_length;
      LOG_MSG("extend "<<extend);
      
      float accu = n_start;
      cur->pho_output.push_back(accu);
      for(int i=0;i<cur->pho_lengths.size();i++)
      {
          float length2 = cur->pho_lengths[i];
          if(cur->pho_types[i]=="VOWEL")
          {
              length2 += extend/vowel_count;
          }
          accu += length2;
          cur->pho_output.push_back(accu);
      }
  }
  else
  {
      float compress = def_length/n_len;
      LOG_MSG("compress "<<compress);
      float accu = n_start;
      cur->pho_output.push_back(accu);
      int size=cur->pho_lengths.size();
      for(int i=0;i<size;i++)
      {
          float length2 = cur->pho_lengths[i];
          length2 -= compress/size;
          LOG_MSG("compress2 "<<length2);
          accu += length2;
          cur->pho_output.push_back(accu);
      }
  }
  
  int count1 = cur->pho_times.size();
  int count2 = cur->pho_output.size();
  
  if(1)  // espeak-sg
  {
  
        auto valid = cur->params.count("FileName");
  
        if (valid && n_len > 0 && count1==count2) {
        LOG_MSG("emit: "<<count2);
        synth->addUnit(cur->params["FileName"], count2, cur->pho_output.data(), cur->pho_times.data());
  
        }
  }
  else
  {
      //emit mbrola/festival commands
      //get diphones from sg-data
      //for each diphone ....
      // get diphones from cur and next
      // cur/cur cur/cur cur/next
      // set context for next
  }
  

}



bool LyricTimelineESPEAK::isValid(){
    return true;
}

void LyricTimelineESPEAK::outputPho(IUnisyn* synth)
{ 
    //first query output
    for(int i=0;i<points.size();i++)
    {
         auto p = points[i];
         auto fileName = p->params["FileName"];
         float length = synth->getLengthForUnit(fileName);
         
         for(int j=0;j<256;j++)
         {
             std::string value = synth->getPhoLine(fileName,j);
             if(value.length()==0) break;
             std::string key = pho(j);
             p->params[key]=value;
             
             
         }
         
         
         if(length>0) p->paramsf["Length"]=length;
         
    }
    
    LyricTimeline::outputPho(synth);
    
}


}
